"use strict"; /**
               * 生成babelrc配置文件，并更新.babelrc，供依赖 babel-loader 的非js文件解析使用
               */
let path = require('path');
let fs = require('fs');
let browsers = require('./browsers');

const defaultPlugins = toCommonjs => {
  return [
  require.resolve("@babel/plugin-syntax-dynamic-import"), // 动态import
  require.resolve("@babel/plugin-syntax-import-meta"),
  ...(toCommonjs ? [require.resolve("@babel/plugin-transform-modules-commonjs")] : []),
  [require.resolve("@babel/plugin-transform-runtime"), { // 运行时帮助函数库
    "corejs": 3 }]];


};

const babelrc = (config, isDebug) => {
  let {
    presets = [], plugins: userConfigPlugins = [] } =
  config.babelrc || {};
  return {
    // 参考 https://github.com/gmfe/Think/issues/67
    // 在wabpack中同一模块下，import 语句不能和 module.exports 同时使用 (babel文档 webpack相关issue) https://github.com/webpack/webpack/issues/4039#issuecomment-419284940
    sourceType: "unambiguous", // 自动推断编译的模块类型
    presets: [
    [require.resolve("@babel/preset-env"), {
      "targets": {
        // https://github.com/ai/browserslist http://browserl.ist/
        "browsers": browsers.getBrowserList(config, isDebug) },

      // 开启modules=false可以使用webpack2的treeshaking，但是，会不兼容amd cmd等模块
      // './lib/kEvent', './lib/react-tabs-redux-custom', './apps/utils/global'
      "modules": false,
      "useBuiltIns": "usage",
      "corejs": 3,
      "debug": false }]].

    concat(presets),
    plugins: defaultPlugins(isDebug).concat(userConfigPlugins) };

};

const bablerc4Server = config => {
  let {
    presets = [], plugins: userConfigPlugins = [] } =
  config.babelrc || {};
  return {
    presets: [
    [require.resolve("@babel/preset-env"), {
      "targets": {
        "node": "current" },

      "useBuiltIns": "usage",
      "corejs": 3 }]].

    concat(presets),
    plugins: defaultPlugins(true).concat(userConfigPlugins) };

};

const update = (config, isDebug) => {
  let babelpath = path.resolve(process.cwd(), './.babelrc');
  let newBabelrc = JSON.stringify(babelrc(config, isDebug), null, 4);
  if (!fs.existsSync(babelpath) || fs.readFileSync(babelpath, 'utf8') !== newBabelrc) {
    fs.writeFileSync(babelpath, newBabelrc);
  }
};

module.exports = {
  update,
  /**
           * babel-loader参数容易配置
           */
  dllOptions(config) {
    return {
      // https://github.com/babel/babel-loader#options
      cacheDirectory: true,
      // https://babeljs.io/docs/usage/options/
      babelrc: false,
      ...babelrc(config, false) };

  },
  /**
      * babel-loader参数容易配置
      */
  clientOptions(config, DEBUG) {
    return {
      // https://github.com/babel/babel-loader#options
      cacheDirectory: true,
      // https://babeljs.io/docs/usage/options/
      babelrc: false,
      ...babelrc(config, DEBUG) };

  },
  /**
      * webpack-server-config用于node端的 babelrc
      */
  serverOptions(config) {
    return {
      // https://github.com/babel/babel-loader#options
      cacheDirectory: true,
      // https://babeljs.io/docs/usage/options/
      babelrc: false,
      ...bablerc4Server(config) };

  } };